import os
from typing import Union


class Configuration:
    def __init__(self, path: str):
        self.path: str = path
        self.src: str = None
        self.dst: str = None
        self.base_url: str = None
        self.title: str = None
        self.dformat: str = None
        self.l_dformat: str = None
        self.lsep_dformat: str = None
        self.force: bool = None


    def read(self):
        try:
            lines: list[str] = None
            with open(self.path, 'r') as f:
                lines = f.readlines()

            opts: dict[str, Union[str, bool]] = dict()
            for l in lines:
                kv: list[str] = l.split('=', 1)
                if len(kv) != 2:
                    raise Exception('wrong config syntax')

                k: str = kv[0].strip()
                v_temp: str = kv[1].strip()
                # check if value should be a boolean true
                v: Union[str, bool] = v_temp\
                    if v_temp.lower() not in ['true', '1', 'yes']\
                    else True

                opts[k] = v

            try:
                self.src = opts['SRC_PATH']
            except KeyError: pass

            try:
                self.dst = opts['DST_PATH']
            except KeyError: pass

            try:
                self.base_url = opts['BASE_URL']
            except KeyError: pass

            try:
                self.title = opts['TITLE']
            except KeyError: pass

            try:
                self.dformat = opts['DATE_FORMAT']
            except KeyError: pass

            try:
                self.l_dformat = opts['LIST_DATE_FORMAT']
            except KeyError: pass

            try:
                self.lsep_dformat = opts['LIST_SEP_DATE_FORMAT']
            except KeyError: pass

            try:
                # if the parser above didn't read a boolean true, then take it
                # as a false anyways
                self.force = opts['FORCE'] if opts['FORCE'] is True else False
            except KeyError: pass

        except OSError: pass


    def fill_missing(self, opts: dict[str, Union[str, bool]]) -> None:
        if self.src is None:
            self.src = opts['src']

        if self.dst is None:
            self.dst = opts['dst']

        if self.base_url is None:
            self.base_url = opts['url']

        if self.title is None:
            self.title = opts['title']

        if self.dformat is None:
            self.dformat = opts['date_format']

        if self.l_dformat is None:
            self.l_dformat = opts['list_date_format']

        if self.lsep_dformat is None:
            self.lsep_dformat = opts['list_sep_date_format']

        if self.force is None:
            self.force = opts['force']



import os
from argparse import ArgumentParser, Namespace
from typing import Union

from .configuration import Configuration
from .database import Database
from .template import Template
from .builder import HTMLBuilder
from .page import Page
from .rss import RSSBuilder


def get_options() -> Namespace:
    parser = ArgumentParser(prog='pyssg',
                            description='''Static Site Generator that reads
                            Markdown files and creates HTML files.\nIf
                            [-c]onfig file is provided (or exists in default
                            location) all other options are ignored.\nFor
                            datetime formats see:
                            https://docs.python.org/3/library/datetime.html#strftime-and-strptime-format-codes''')
    parser.add_argument('-c', '--config',
                        default='$XDG_CONFIG_HOME/pyssg/pyssgrc',
                        type=str,
                        help='''config file (path) to read from; defaults to
                        '$XDG_CONFIG_HOME/pyssg/pyssgrc' ''')
    parser.add_argument('-s', '--src',
                        default='src',
                        type=str,
                        help='''src directory; handmade files, templates and
                        metadata directory; defaults to 'src' ''')
    parser.add_argument('-d', '--dst',
                        default='dst',
                        type=str,
                        help='''dst directory; generated (and transfered html)
                        files; defaults to 'dst' ''')
    parser.add_argument('-u', '--url',
                        default='',
                        type=str,
                        help='''base url without trailing slash''')
    parser.add_argument('-t', '--title',
                        default='Blog',
                        type=str,
                        help='''general title for the website; defaults to
                        'Blog' ''')
    parser.add_argument('--date-format',
                        default='%a, %b %d, %Y @ %H:%M %Z',
                        type=str,
                        help='''date format used inside pages (for creation and
                        modification times, for example); defaults to '%%a, %%b
                        %%d, %%Y @ %%H:%%M %%Z' ('Tue, Mar 16, 2021 @ 02:46 UTC',
                        for example)''')
    parser.add_argument('--list-date-format',
                        default='%b %d',
                        type=str,
                        help='''date format used for page entries in a list;
                        defaults to '%%b %%d' ('Mar 16', for example)''')
    parser.add_argument('--list-sep-date-format',
                        default='%B %Y',
                        type=str,
                        help='''date format used for the separator between page
                        entries in a list; defaults to '%%B %%Y' ('March 2021',
                        for example)''')
    parser.add_argument('-i', '--init',
                        action='store_true',
                        help='''initializes the dir structure, templates,
                        as well as the 'src' and 'dst' directories''')
    parser.add_argument('-b', '--build',
                        action='store_true',
                        help='''generates all html files and passes over
                        existing (handmade) ones''')
    parser.add_argument('-f', '--force',
                        action='store_true',
                        help='''force building all pages and not only the
                        updated ones''')

    return parser.parse_args()


def main() -> None:
    opts: dict[str, Union[str, bool]] = vars(get_options())
    conf_path: str = opts['config']
    conf_path = os.path.expandvars(conf_path)

    config: Configuration = Configuration(conf_path)
    config.read()
    config.fill_missing(opts)

    if opts['init']:
        try:
            os.mkdir(config.src)
            os.makedirs(os.path.join(config.dst, 'tag'))
        except FileExistsError:
            pass

        # write default templates
        template: Template = Template(config.src)
        template.write()
        return

    if opts['build']:
        # start the db
        db: Database = Database(os.path.join(config.src, '.files'))
        db.read()

        # read templates
        template: Template = Template(config.src)
        template.read()

        builder: HTMLBuilder = HTMLBuilder(config, template, db)
        builder.build()

        # get all parsed pages for rss construction
        all_pages: list[Page] = builder.get_pages()
        rss_builder: RSSBuilder = RSSBuilder(template.rss, all_pages)
        rss_builder.build()

        db.write()
        return



import os

from .page import Page


# all objects here require a header and footer as minimum
class HF:
    def __init__(self):
        self.header: str = None
        self.footer: str = None


# some objects require a "list-like" set of attributes
class Common(HF):
    def __init__(self):
        self.list_header: str = None
        self.list_footer: str = None
        self.list_entry: str = None
        self.list_separator: str = None


# main class
class Template(HF):
    def __init__(self, src: str):
        self.src: str = src
        self.article: HF = HF()
        self.articles: Common = Common()
        self.tags: Common = Common()
        self.rss: str = None

        self.is_read: bool = False


    # writes default templates
    def write(self) -> None:
        # get initial working directory
        iwd = os.getcwd()
        os.chdir(self.src)

        # create templates dir
        os.mkdir('templates')
        os.chdir('templates')

        # common
        os.mkdir('common')
        os.chdir('common')
        self.__write_template('header.html',
                              ['<!DOCTYPE html>\n',
                               '<html lang="$$LANG">\n',
                               '<head>\n',
                               '<meta charset="utf-8">\n',
                               '<title>$$TITLE</title>\n',
                               '</head>\n',
                               '<body>\n'])
        self.__write_template('footer.html',
                              ['</body>\n',
                               '</html>\n'])

        # go back to templates
        os.chdir('..')

        # article entry
        os.mkdir('article')
        os.chdir('article')
        self.__write_template('header.html',
                              ['<h1>$$TITLE</h1>\n',
                               '<p>$$AUTHOR</p>\n',
                               '<p>Created: $$CTIME, modified: $$MTIME</p>\n'])
        self.__write_template('footer.html',
                              [''])

        # go back to templates
        os.chdir('..')

        # article index (articles)
        os.mkdir('articles')
        os.chdir('articles')
        self.__write_template('header.html',
                              [''])
        self.__write_template('list_header.html',
                              ['<h2>Articles</h2>\n',
                               '<ul>\n'])
        self.__write_template('list_entry.html',
                              ['<li>$$DATE - <a href="$$URL">$$TITLE</a></li>\n'])
        self.__write_template('list_separator.html',
                              ['<h3>$$SEP</h3>\n'])
        self.__write_template('list_footer.html',
                              ['</ul>\n'])
        self.__write_template('footer.html',
                              [''])

        # go back to templates
        os.chdir('..')

        # tag
        os.mkdir('tag')
        os.chdir('tag')
        self.__write_template('header.html',
                              [''])
        self.__write_template('list_header.html',
                              ['<p>Tags: '])
        self.__write_template('list_entry.html',
                              ['<a href="$$URL">$$NAME</a>'])
        self.__write_template('list_separator.html',
                              [', '])
        self.__write_template('list_footer.html',
                              ['</p>\n'])
        self.__write_template('footer.html',
                              [''])

        # go back to templates
        os.chdir('..')

        os.mkdir('rss')
        os.chdir('rss')
        self.__write_template('rss.xml',
                              ['<?xml version="1.0" encoding="UTF-8" ?>\n',
                               '<rss version="2.0">\n',
                               '  <channel>\n',
                               '    <title>$$TITLE</title>\n',
                               '    <link>$$LINK</link>\n',
                               '    <atom:link href="EXAMPLE.ORG/RSS.XML" rel="self" type="application/rss+xml"/>\n',
                               '    <description>SHORT DESCRIPTION.</description>\n',
                               '    <language>en-us</language>\n',
                               '    <copyright>COPYRIGHT NOTICE.</copyright>\n',
                               '    <managingEditor>EMAIL@EXAMPLE.ORG</managingEditor>\n',
                               '    <webMaster>EMAIL@EXAMPLE.ORG</webMaster>\n',
                               '    <pubDate>$$CURRENTDATE</pubDate>\n',
                               '    <lastBuildDate>$$CURRENTDATE</lastBuildDate>\n',
                               '    <generator>$$PYSSGVERSION</generator>\n',
                               '    <docs>https://validator.w3.org/feed/docs/rss2.html</docs>\n',
                               '    <ttl>30</ttl>\n',
                               '    <image>\n',
                               '      <url>EXAMPLE.ORG/IMAGE.PNG</url>\n',
                               '      <title>$$TITLE</title>\n',
                               '      <link>$$LINK</link>\n',
                               '    </image>\n',
                               '$$ITEMS\n',
                               '  </channel>\n',
                               '</rss>'])

        # return to initial working directory
        os.chdir(iwd)


    # reads templates and stores them into class attributes
    def read(self) -> None:
        # only read templates if not read already
        # (might want to change this behaviour)
        if self.is_read:
            return
        self.is_read = True

        # get initial working directory
        iwd = os.getcwd()
        os.chdir(os.path.join(self.src, 'templates'))

        # common
        os.chdir('common')
        self.header = self.__read_template('header.html')
        self.footer = self.__read_template('footer.html')

        # go back to templates
        os.chdir('..')

        # article entry
        os.chdir('article')
        self.article.header = self.__read_template('header.html')
        self.article.footer = self.__read_template('footer.html')

        # go back to templates
        os.chdir('..')

        # article index
        os.chdir('articles')
        self.articles.header = self.__read_template('header.html')
        self.articles.list_header = \
                self.__read_template('list_header.html')
        self.articles.list_entry = \
                self.__read_template('list_entry.html')
        self.articles.list_separator = \
                self.__read_template('list_separator.html')
        self.articles.list_footer = \
                self.__read_template('list_footer.html')
        self.articles.footer = self.__read_template('footer.html')

        # go back to templates
        os.chdir('..')

        # tag
        os.chdir('tag')
        self.tags.header = self.__read_template('header.html')
        self.tags.list_header = self.__read_template('list_header.html')
        self.tags.list_entry = self.__read_template('list_entry.html')
        self.tags.list_separator = self.__read_template('list_separator.html')
        self.tags.list_footer = self.__read_template('list_footer.html')
        self.tags.footer = self.__read_template('footer.html')

        # go back to templates
        os.chdir('..')

        # tag
        os.chdir('rss')
        self.rss = self.__read_template('rss.xml')

        # return to initial working directory
        os.chdir(iwd)


    def __write_template(self, file_name: str, content: list[str]) -> None:
        with open(file_name, 'w+') as f:
            for c in content:
                f.write(c)

    def __read_template(self, file_name: str) -> str:
        out: str = None
        with open(file_name, 'r') as f:
            out = f.read()

        return out



from datetime import datetime, timezone


class Page:
    def __init__(self,
                 name: str,
                 c_time: float,
                 m_time: float,
                 html: str,
                 meta: dict):
        self.name: str = name
        self.c_time: float = c_time
        self.m_time: float = m_time
        self.html: str = html
        self.meta: dict = meta

        # data from self.meta
        self.title: str = ''
        self.author: str = ''
        self.c_datetime: datetime = None
        self.m_datetime: datetime = None
        self.summary: str = ''
        self.lang: str = 'en'
        self.tags: list = None

        # also from self.meta, but for og metadata
        self.og: dict[str, str] = dict()

        self.__parse_meta()

    def __lt__(self, other):
        return self.c_time < other.c_time


    # parses meta from self.meta, for og, it prioritizes,
    # the actual og meta
    def __parse_meta(self):
        try:
            self.title = self.meta['title'][0]
        except KeyError: pass

        try:
            self.author = self.meta['author'][0]
        except KeyError: pass

        self.c_datetime = datetime.fromtimestamp(self.c_time,
                                                 tz=timezone.utc)

        if self.m_time != 0.0:
            self.m_datetime = datetime.fromtimestamp(self.m_time,
                                                     tz=timezone.utc)

        try:
            self.summary = self.meta['summary'][0]
        except KeyError: pass

        try:
            self.lang = self.meta['lang'][0]
        except KeyError: pass

        try:
            self.tags = self.meta['tags']
            self.tags.sort()
        except KeyError: pass

        try:
            # og_e = object graph entry
            for og_e in self.meta['og']:
                kv: str = og_e.split(',', 1)
                if len(kv) != 2:
                    raise Exception('invalid og syntax')

                k: str = kv[0].strip()
                v: str = kv[1].strip()

                self.og[k] = v
        except KeyError: pass



import os
import importlib.metadata
from datetime import datetime, timezone

from .page import Page
from .configuration import Configuration


VERSION = importlib.metadata.version('pyssg')
DFORMAT = '%a, %d %b %Y %H:%M:%S %Z'


class RSSBuilder:
    def __init__(self, config: Configuration,
                 template: str,
                 pages: list[Page]):
        self.rss: str = template
        self.pages: list[Page] = pages


    def build(self):
        # initial base replacements
        self.rss = self.rss.replace('$$TITLE', config.title)
        self.rss = self.rss.replace('$$LINK', config.base_url)
        self.rss = self.rss.replace('$$PYSSGVERSION', VERSION)
        items_formatted: str = __get_items_formatted()
        self.rss = self.rss.replace('$$ITEMS', items_formatted)

        current_date: str = datetime.now(tz=timezone.utc).strftime(DFORMAT)
        self.rss = self.rss.replace('$$CURRENTDATE', current_date)

        with open(os.path.join(config.dst, 'rss.xml'), 'w') as f:
            f.write(self.rss)


    def __get_items_formatted(self) -> str:
        # i_f=items formatted for short
        i_f: str = ''
        for p in pages:
            url: str = f'{config.base_url}/{p.name.replace(".md", ".html")}'
            date: str = p.c_datetime.strftime(DFORMAT)

            i_f = f'{i_f}    <item>\n'
            i_f = f'{i_f}      <title>{p.title}</title>\n'
            i_f = f'{i_f}      <link>{url}</link>\n'
            i_f = f'{i_f}      <description>{p.summary}</description>\n'
            i_f = f'{i_f}      <guid isPermaLink="true">{url}</guid>\n'
            i_f = f'{i_f}      <pubDate>{date}</pubDate>\n'
            i_f = f'{i_f}    </item>\n'

        return i_f



import os
from datetime import datetime
from markdown import Markdown

from .database import Database
from .page import Page


# parser of md files, stores list of pages and tags
class MDParser:
    def __init__(self, src: str,
                 files: list[str],
                 db: Database):
        self.md: Markdown = Markdown(extensions=['extra', 'meta', 'sane_lists',
                                                 'smarty', 'toc', 'wikilinks'],
                                     output_format='html5')
        self.src: str = src
        self.files: list[str] = files
        self.db: Database = db

        self.all_pages: list[Page] = None
        self.updated_pages: list[Page] = None
        self.all_tags: list[str] = None


    def parse(self):
        # initialize lists
        self.all_pages = []
        self.updated_pages = []
        self.all_tags = []

        for f in self.files:
            src_file: str = os.path.join(self.src, f)
            # get flag if update is successful
            updated: bool = self.db.update(src_file, remove=f'{self.src}/')

            page: Page = None
            content: str = self.md.reset().convert(open(src_file).read())
            page = Page(f, self.db.e[f][0], self.db.e[f][1], content, self.md.Meta)

            # keep a separated list for all and updated pages
            if updated:
                self.updated_pages.append(page)
            self.all_pages.append(page)

            # parse tags
            if page.tags is not None:
                # add its tag to corresponding db entry if existent
                self.db.update_tags(f, page.tags)

                # update all_tags attribute
                for t in page.tags:
                    if t not in self.all_tags:
                        self.all_tags.append(t)

        # sort list of tags for consistency
        self.all_tags.sort()
        self.updated_pages.sort(reverse=True)
        self.all_pages.sort(reverse=True)



import os
import shutil
from copy import deepcopy

from .configuration import Configuration
from .template import Template
from .database import Database
from .parser import MDParser
from .page import Page
from .discovery import get_file_list, get_dir_structure

class HTMLBuilder:
    def __init__(self, config: Configuration,
                 template: Template,
                 db: Database):
        self.src: str = config.src
        self.dst: str = config.dst
        self.base_url: str = config.base_url
        self.dformat: str = config.dformat
        self.l_dformat: str = config.l_dformat
        self.lsep_dformat: str = config.lsep_dformat
        self.force: bool = config.force

        self.template: Template = template
        self.db: Database = db

        self.dirs: list[str] = None
        self.md_files: list[str] = None
        self.html_files: list[str] = None

        self.all_pages: list[Page] = None


    def build(self) -> None:
        self.dirs = get_dir_structure(self.src, ['templates'])
        self.md_files = get_file_list(self.src, ['.md'], ['templates'])
        self.html_files = get_file_list(self.src, ['.html'], ['templates'])

        self.__create_dir_structure()
        self.__copy_html_files()

        parser: MDParser = MDParser(self.src, self.md_files, self.db)
        parser.parse()

        # just to be able to extract all pages out of this class
        self.all_pages = parser.all_pages

        # create the article index
        self.__create_article_index(parser.all_tags, parser.all_pages)

        # create each category of html pages
        # check if all pages should be created
        if self.force:
            self.__create_articles(parser.all_pages)
        else:
            self.__create_articles(parser.updated_pages)
        self.__create_tags(parser.all_tags, parser.all_pages)


    def get_pages(self) -> list[Page]:
        return self.all_pages


    def __create_dir_structure(self) -> None:
        for d in self.dirs:
            # for the dir structure,
            # doesn't matter if the dir already exists
            try:
                os.makedirs(os.path.join(self.dst, d))
            except FileExistsError:
                pass


    def __copy_html_files(self) -> None:
        src_file: str = None
        dst_file: str = None

        for f in self.html_files:
            src_file = os.path.join(self.src, f)
            dst_file = os.path.join(self.dst, f)

            # only copy files if they have been modified (or are new)
            if self.db.update(src_file, remove=f'{self.src}/'):
                shutil.copy2(src_file, dst_file)


    # this is really similar to create_tag (singular)
    def __create_article_index(self, tags: list[str],
                               pages: list[Page]) -> None:
        # make temporary template
        t: Template = deepcopy(self.template)

        # do basic replacements
        # get page and tag list formated, both functions do replacements
        p_list: list[str] = self.__get_pages_formatted(pages, t)
        t_list: list[str] = self.__get_tags_formatted(tags, t)
        # common
        t.header = t.header.replace("$$LANG", 'en')
        t.header = t.header.replace('$$TITLE', f'Index')

        with open(os.path.join(self.dst, 'index.html'), 'w') as f:
            f.write(t.header)
            f.write(t.articles.header)

            f.write(t.tags.list_header)
            for tag in t_list:
                f.write(tag)
            f.write(t.tags.list_footer)

            f.write(t.articles.list_header)
            for page in p_list:
                f.write(page)
            f.write(t.articles.list_footer)

            f.write(t.articles.footer)
            f.write(t.footer)


    def __create_articles(self, pages: list[Page]) -> None:
        for p in pages:
            self.__create_article(p)


    def __create_article(self, page: Page) -> None:
        # TODO: create better solution for replace
        # make temporary template
        t: Template = deepcopy(self.template)

        # prepare html file name
        f_name: str = page.name
        f_name = f_name.replace('.md', '.html')

        # get timestamps
        c_date: str = page.c_datetime.strftime(self.dformat)
        m_date: str = None
        if page.m_datetime is not None:
            m_date: str = page.m_datetime.strftime(self.dformat)

        # do basic replacements
        # get tag list formatted (some replacements done inside
        # get_tags_formatted)
        t_list: list[str] = None
        if page.tags is not None:
            t_list = self.__get_tags_formatted(page.tags, t)

        # common
        t.header = t.header.replace("$$LANG", page.lang)
        t.header = t.header.replace('$$TITLE', page.title)

        # article header
        t.article.header = t.article.header.replace('$$TITLE', page.title)
        t.article.header = t.article.header.replace('$$AUTHOR', page.author)
        t.article.header = t.article.header.replace('$$CTIME', c_date)
        if m_date is not None:
            t.article.header = t.article.header.replace('$$MTIME', m_date)
        else:
            t.article.header = t.article.header.replace('$$MTIME', '')

        # article footer (same replaces as header)
        t.article.footer = t.article.footer.replace('$$TITLE', page.title)
        t.article.footer = t.article.footer.replace('$$AUTHOR', page.author)
        t.article.footer = t.article.footer.replace('$$CTIME', c_date)
        if m_date is not None:
            t.article.footer = t.article.footer.replace('$$MTIME', m_date)
        else:
            t.article.footer = t.article.footer.replace('$$MTIME', '')


        with open(os.path.join(self.dst, f_name), 'w') as f:
            f.write(t.header)
            f.write(t.article.header)
            f.write(page.html)

            if t_list is not None:
                f.write(t.tags.list_header)
                for tag in t_list:
                    f.write(tag)
                f.write(t.tags.list_footer)

            f.write(t.article.footer)
            f.write(t.footer)


    def __get_tags_formatted(self, tags: list[str],
                             template: Template) -> list[str]:
        tag_amount: int = len(tags)
        tags_formatted: list[str] = []
        for i, t in enumerate(tags):
            # t_e=tag entry
            t_e: str = template.tags.list_entry
            t_e = t_e.replace('$$URL',
                              f'{self.base_url}/tag/@{t}.html')
            t_e = t_e.replace('$$NAME', t)

            tags_formatted.append(t_e)
            if i != tag_amount - 1:
                tags_formatted.append(template.tags.list_separator)

        return tags_formatted


    def __create_tags(self, tags: list[str],
                      pages: list[Page]) -> None:
        for t in tags:
            # get a list of all pages that have current tag
            tag_pages: list[Page] = []
            for p in pages:
                if p.tags is not None and t in p.tags:
                    tag_pages.append(p)

            # build tag page
            self.__create_tag(t, tag_pages)

            # clean list of pages with current tag
            tag_pages = []


    def __create_tag(self, tag: str,
                     pages: list[Page]) -> None:
        # TODO: create better solution for replace
        # make temporary template
        t: Template = deepcopy(self.template)

        # do basic replacements
        # get page list formated (some replacements done inside
        # get_pages_formatted)
        p_list: list[str] = self.__get_pages_formatted(pages, t)
        # common
        t.header = t.header.replace("$$LANG", 'en')
        t.header = t.header.replace('$$TITLE', f'Posts filtered by: {tag}')

        # tag header
        tag_url: str = f'{self.base_url}/tag/@{tag}.html'
        t.tags.header = t.tags.header.replace('$$NAME', tag)

        with open(os.path.join(self.dst, f'tag/@{tag}.html'), 'w') as f:
            f.write(t.header)
            f.write(t.tags.header)

            f.write(t.articles.list_header)
            for p in p_list:
                f.write(p)
            f.write(t.articles.list_footer)

            f.write(t.tags.footer)
            f.write(t.footer)


    def __get_pages_formatted(self, pages: list[Page],
                              template: Template) -> list[str]:
        month_year: str = '-'
        pages_formatted: list[str] = []
        for p in pages:
            # check if the monthly separator should be included
            c_month_year: str = p.c_datetime.strftime(self.lsep_dformat)
            if c_month_year != month_year:
                month_year = c_month_year

                month_sep: str = template.articles.list_separator
                month_sep = month_sep.replace('$$SEP', month_year)

                pages_formatted.append(month_sep)

            f_name: str = p.name
            f_name = f_name.replace('.md', '.html')

            # p_e=page entry
            p_e: str = template.articles.list_entry
            p_e = p_e.replace('$$URL', f'{self.base_url}/{f_name}')
            p_e = p_e.replace('$$DATE', p.c_datetime.strftime(self.l_dformat))
            p_e = p_e.replace('$$TITLE', p.title)

            pages_formatted.append(p_e)

        return pages_formatted
